Device = 18F2550         // Tell the compiler what chip we are using
Clock = 8                // Tell the compiler what we will be setting the clock to (Mhz)           
Config FOSC= INTOSCIO_EC     // This tells the microcontroller to use the internal clock 
Config MCLRE = OFF

Include "utils.bas"

Const BallSound(10) As Word = (15,1899,15,1691,15,1505,15,500,15,1000) 
Const Notes(12) As Word = (1899,1791,1691,1595,1505,1420,1340,1264,1192,1124,1061,1001)
Const MenuSound(6) As Word = (30,1600,50,1300,50,1000) 
Const WinnerMusic(12) As Word = (50,1400,50,1300,100,1350,100,1200,150,900,50,902) 
'Const WinnerMusic(14) As Word = (70,1400,75,1350,80,1300,85,1250,90,1200,180,1100,70,1000) 
Const BallOutOfBoundsSound(6) As Word = (20,2000,20,2300,20,2600)
 
Dim RBIF As INTCON.0
Dim RBIE As INTCON.3
Dim DoBallSound As Boolean 
Dim DoMenuSound As Boolean
Dim DoWinnerMusic As Boolean
Dim DoBallOutOfBoundsSound As Boolean
Dim x As Byte
Dim TempSound As Word
Dim TempDelay As Word
Dim WinnerMusicUp As Boolean

 
Interrupt PORTB_Change()
    Save(0) // Save the system variables
    If RBIF = 1 Then // Check if the interrupt was a PORTB change
        RBIF = 0
        If PORTB.bits(7) = 1 Then   ' did the interrupt come from PORTB pin 7? (which is our play sound connection)
            x = 0
            ' stop all sounds, then figure out which one to play
            DoMenuSound = false
            DoBallSound = false
            DoBallOutOfBoundsSound = false
            DoWinnerMusic = false
            If PORTB.bits(3) = 0 And PORTB.bits(2) = 0 Then
                DoMenuSound = true
                TempDelay = MenuSound(0)    ' the first part of the array is the delay (then the frequency)
            ElseIf PORTB.bits(3) = 0 And PORTB.bits(2) = 1 Then 
                DoBallSound = true
                TempDelay = BallSound(0)    ' the first part of the array is the delay (then the frequency) 
            ElseIf PORTB.bits(3) = 1 And PORTB.bits(2) = 0 Then  
                DoBallOutOfBoundsSound = true
                TempDelay = BallOutOfBoundsSound(0)    ' the first part of the array is the delay (then the frequency)
            ElseIf PORTB.bits(3) = 1 And PORTB.bits(2) = 1 Then  
                DoWinnerMusic = true
                TempDelay = WinnerMusic(0)  ' may need to incorporate some winner music delays in with the music...
            EndIf
        EndIf
    EndIf
    Restore // Restore the system variables
End Interrupt
 
Sub PORTB_Interrupts(Control As Boolean) // Small routine to enable/disable PORTB Interrupts.
    If Control = True Then //
        RBIE = 1 //
        Enable(PORTB_Change) //
    Else //
        RBIE = 0 //
        Disable(PORTB_Change) //
    EndIf //
End Sub

Sub PlayMenuSound()
    If DoMenuSound = true Then
        TempSound = MenuSound(x + 1)    ' the actual sound (or frequency) comes from each second part of the array (the first part is the delay)
        If TempDelay <> 0 Then
            Dec(TempDelay)
            PORTB.bits(0) = 1
            DelayUS(TempSound)
            PORTB.bits(0) = 0
            DelayUS(TempSound)
        Else
            TempDelay = MenuSound(x)
            x = x + 2
            If x > Bound(MenuSound) Then
                DoMenuSound = false
            EndIf
        EndIf
    EndIf
End Sub
      
Sub PlayBallSound()
    If DoBallSound = true Then
        TempSound = BallSound(x + 1) 
        If TempDelay <> 0 Then
            Dec(TempDelay)
            PORTB.bits(0) = 1
            DelayUS(TempSound)
            PORTB.bits(0) = 0
            DelayUS(TempSound)
        Else
            TempDelay = BallSound(x)
            x = x + 2
            If x > Bound(BallSound) Then
                DoBallSound = false
            EndIf
        EndIf
    EndIf
End Sub

Sub PlayBallOutOfBoundsSound()
    If DoBallOutOfBoundsSound = true Then
        TempSound = BallOutOfBoundsSound(x + 1) 
        If TempDelay <> 0 Then
            Dec(TempDelay)
            PORTB.bits(0) = 1
            DelayUS(TempSound)
            PORTB.bits(0) = 0
            DelayUS(TempSound)
        Else
            TempDelay = BallOutOfBoundsSound(x)
            x = x + 2
            If x > Bound(BallOutOfBoundsSound) Then
                DoBallOutOfBoundsSound = false
            EndIf
        EndIf
    EndIf
End Sub

Sub PlayWinnerMusic()
    If DoWinnerMusic = true Then
        TempSound = WinnerMusic(x + 1) 
        If TempDelay <> 0 Then
            Dec(TempDelay)
            PORTB.bits(0) = 1
            DelayUS(TempSound)
            PORTB.bits(0) = 0
            DelayUS(TempSound)
        Else
            TempDelay = WinnerMusic(x)
            x = x + 2
            If x > Bound(WinnerMusic) Then
                DoWinnerMusic = false
                x = 0
            EndIf
        EndIf
    EndIf
End Sub 
      
(*Sub PlayWinnerMusic()
    If DoWinnerMusic = true Then
        TempSound = Notes(x) 
        If TempDelay <> 0 Then
            Dec(TempDelay)
            PORTB.bits(0) = 1
            DelayUS(TempSound)
            PORTB.bits(0) = 0
            DelayUS(TempSound)
        Else
            TempDelay = 20
            If WinnerMusicUp = true Then
                x = x + 1
                If x > Bound(Notes) Then
                    WinnerMusicUp = false
                    'DoWinnerMusic = false
                    x = x - 2  ' go back to the previous note (going back by one will go back to the same note)
                EndIf
            ElseIf WinnerMusicUp = false Then
                x = x - 1
                If x = 255 Then
                    WinnerMusicUp = true
                    'DoWinnerMusic = false
                    x = x + 2  ' go back to the previous note (going back by one will go back to the same note)
                EndIf
            EndIf
        EndIf
    EndIf
End Sub*)

 
// Start Of Program...
OSCCON = %01111100
SetAllDigital
TRISB = %10001100 // Make PORTB Pin 7 an Input (this is the interrupt) and also make PORTB bits 2 and 3 inputs (these are our audio select lines)
 
PORTB_Interrupts(true) // Enable PORTB interrupts
DoBallSound = false
DoMenuSound = false
DoBallOutOfBoundsSound = false
DoWinnerMusic = false
TempDelay = WinnerMusic(0)
WinnerMusicUp = true
 
While True // Infinite loop
    PlayBallSound
    PlayMenuSound
    PlayBallOutOfBoundsSound
    PlayWinnerMusic
Wend
